" Kuka Robot Language indent file for Vim
" Language: Kuka Robot Language
" Maintainer: Patrick Knosowski <knosowski@graeff.de>
" Version: 0.0.7
" Last Change:
" Credits: Based on indent/vim.vim
"
" Suggestions of improvement are very welcome. Please email me!
"
" Known bugs: 1. If a subroutine is named so that it starts with a keyword that
"             will decrease the level of indention, then the call is indeted
"             accordingly. You can fix this by hitting ^F while still in
"             insert mode after you finished the complete call, or use ^P/^N
"             before the keyword is complete which causes the indention.
"
"             Keywords that decrease the level of indention are:
"             end*, else, case, default, until and continue. The 'continue' only
"             decreases if the next line is one of the other mentioned keywords.
"
"             Example: If a subroutine is called CaseHandling1() you
"             must either type:
"             CaseHandling1()^F
"             or:
"             Cas^P()
"             The latter only works if the name already exist anywhere in the
"             files you have in buffer.
"
"
"             2. A 'continue' belongs to the following line, so I want it
"             do have the same indention as the following line:
"
"             Example:
"             repeat
"               stuff()
"             continue
"             until $in[1]
"
"             Now, if the following 'until' does not yet exist, 'continue' will be
"             indented like the previous line. ^F in insert mode or = in
"             normal mode on the line will fix this once the 'until' is present.
"             Hint: = can be combined with movement commands.
"             Example: gg=G will indent the complete file.
"
" TODO
" * intensiv testing, so far I tested mostly ^F, but just little typing in
"   insert mode
" * avoid false indention of calls like Endbedingungen(), CaseHandling1() or
"   DefaultValues() ect in insert mode
"

if exists("g:krlNoIndent") && g:krlNoIndent==1
  finish
endif

" Only load this indent file when no other was loaded.
if exists("b:did_indent")
  finish
endif
let b:did_indent = 1

setlocal nolisp
setlocal nosmartindent
setlocal autoindent
setlocal indentexpr=GetKrlIndent()
setlocal indentkeys=!^F,o,O,0=~end,0=~else,0=~case,0=~default,0=~until,0=~continue

let b:undo_indent = "setl indentkeys< indentexpr<"

" Only define the function once.
if exists("*GetKrlIndent")
  finish
endif
let s:keepcpo= &cpo
set cpo&vim

" TODO wenn ich das richtig verstehe setzt das die ignorecase option, aber
" warum? und warum try? kann das weg?
function GetKrlIndent()
  let ignorecase_save = &ignorecase
  try
    let &ignorecase = 0
    return GetKrlIndentIntern()
  finally
    let &ignorecase = ignorecase_save
  endtry
endfunction

function GetKrlIndentIntern()
  " Find a non-blank line above the current line.
  " let lnum = prevnonblank(v:lnum - 1)
  let lnum = KrlPreNoneBlank(v:lnum - 1)

  " At the start of the file use zero indent.
  if lnum == 0
    return 0
  endif

  " Add a 'shiftwidth' after :def*, :if, :else, :case,
  " :default, :for, :while, :repeat and :loop.
  let ind = indent(lnum)
  let line = getline(lnum)
  let i = match(line, '\c\v^\s*
                      \(
                      \(global\s+)?def
                          \(\s+\w
                          \|fct\s+\w
                          \|dat\s+\w
                          \)
                      \|if\W+
                      \|else\s*(;.*)?$
                      \|case\W+
                      \|default\s*(;.*)?$
                      \|for\W+
                      \|while\W+
                      \|repeat\s*(;.*)?$
                      \|loop\s*(;.*)?$
                      \)'
          \)
                      " \|switch\s+
  if i >= 0
    let ind += &sw
  endif

  " Subtract a 'shiftwidth' on a :end*, :else, :case, :default and :until
  if getline(v:lnum) =~ '\c\v^\s*
        \(end(|fct|dat|if|switch|for|while|loop)\s*(;.*)?$
        \|else\s*(;.*)?$
        \|case>
        \|default\s*(;.*)?$
        \|until>
        \)'
    let ind = ind - &sw
  endif

  " first case after a switch
  if getline(v:lnum) =~ '\c\v^\s*case>'
    " let lnum = prevnonblank(v:lnum - 1)
    let lnum = KrlPreNoneBlank(v:lnum - 1)
    " At the start of the file use zero indent.
    if lnum == 0
      return 0
    endif
    if getline(lnum) =~ '\c\v^\s*switch>'
      let ind = ind + &sw
    endif
  endif

  " keep continue-instructions the same indention like the following
  " instruction if its end*, else, case, default or until
  if getline(v:lnum) =~ '\c\v^\s*continue>' && getline(v:lnum + 1) =~ '\c\v^\s*
        \(end(|fct|dat|if|switch|for|while|loop)\s*(;.*)?$
        \|else\s*(;.*)?$
        \|case>
        \|default\s*(;.*)?$
        \|until>
        \)'
    let ind = ind - &sw
  endif

  return ind
endfunction

function KrlPreNoneBlank(lnum)
  " this function handles &foo-headers, comments and continue-instructions like blank lines
  let nPreNoneBlank = prevnonblank(a:lnum)
  " At the start of the file use zero indent.
  if nPreNoneBlank == 0
    return 0
  endif

  let i=1
  while i>=1 && nPreNoneBlank>=0
    if getline(nPreNoneBlank) =~ '\v\c^\s*
          \(\&.*$
          \|(;.*)?$
          \|continue\s*(;.*)?$
          \)'
      let nPreNoneBlank = prevnonblank(nPreNoneBlank - 1)
      " At the start of the file use zero indent.
      if nPreNoneBlank == 0
        return 0
      endif
    else
      let i=0
    endif
  endwhile

  return nPreNoneBlank
endfunction

let &cpo = s:keepcpo
unlet s:keepcpo

" vim:sw=2
