" Kuka Robot Language syntax file for Vim
" Language: Kuka Robot Language
" Maintainer: Patrick Knosowski <knosowski@graeff.de>
" Version: 1.1.3
" Last Change: 22.05.2016
" Credits: Thanks for contributions to this to Michael Jagusch
" 
" Suggestions of improvement are very welcome. Please email me!
"
" TODO: Make folds case insensitive
"

" Remove any old syntax stuff that was loaded (5.x) or quit when a syntax file
" was already loaded (6.x).
if version < 600
  syntax clear
elseif exists("b:current_syntax")
  finish
endif

" krl does ignore case
syn case ignore

" folding
if has("folding")
  setlocal foldmethod=marker
  setlocal foldtext=v:folddashes.substitute(getline(v:foldstart),';\s*FOLD\\\|;[^;]*$','','g')
  if exists("g:krlCloseFolds") && g:krlCloseFolds==1
    " close all folds; too sad, this is case senitive
    setlocal foldmarker=FOLD,ENDFOLD 
  elseif exists("g:krlCloseFolds") && g:krlCloseFolds==2
    " disable folding completely
    " set to vim defaults
    setlocal foldmarker={{{,}}}
    setlocal foldmethod=manual
    setlocal foldtext=foldtext()
  else
    " close only PTP|LIN|CIRC movement folds, also case sensitive
    setlocal foldmarker=%CMOVE,ENDFOLD
  endif
endif

" Comment
" TODO Comment
syn keyword krlTodo contained TODO FIXME XXX
highlight link krlTodo Todo
" Debug Comment
syn keyword krlDebug contained DEBUG
highlight link krlDebug Debug
" Line Comment
syn match krlComment /;.*$/ contains=krlTodo,krlDebug
highlight link krlComment Comment
" ---

" Header
syn match krlHeader /&\a\w*/
highlight link krlHeader PreProc
" ---

" Constant values
" Boolean
syn keyword krlBoolean TRUE FALSE containedin=krlStructVal
highlight link krlBoolean Boolean
" Integer
syn match krlInteger /\W[+-]\?\d\+/lc=1 containedin=krlStructVal
highlight link krlInteger Number
" Binary integer
syn match krlBinaryInt /'b[01]\+'/ containedin=krlStructVal
highlight link krlBinaryInt Number
" Hexadecimal integer
syn match krlHexInt /'h[0-9a-fA-F]\+'/ containedin=krlStructVal
highlight link krlHexInt Number
" Float
syn match krlFloat /\W[+-]\?\d\+\.\d\+\([eE][+-]\?\d\+\)\?/lc=1 containedin=krlStructVal
highlight link krlFloat Float
" String
syn region krlString start=/"/ end=/"/ containedin=krlStructVal
highlight link krlString String
" Enum
syn match krlEnumVal /#\a\w*/ containedin=krlStructVal
highlight link krlEnumVal Constant
" ---

" Structure
" Predefined structures and enums found in
" /r1/mada/$*.dat, /r1/steu/$*.dat and
" /r1/system/$config.dat as well as
" basisTech, gripperTech and spotTech
"
" Predefined structures for movement
syn keyword krlStructure FRAME E6POS POS E6AXIS AXIS
syn keyword krlStructure FDAT LDAT PDAT
syn keyword krlStructure LOAD INERTIA
"
" Predefined structures for shapes
syn keyword krlStructure AXBOX CYLINDER BOX
"
" Predefined structures and enums found in /r1/mada/$machine.dat
syn keyword krlStructure CP FRA ACC_CAR JERK_STRUC DHART SPIN TRPSPIN EX_KIN ET_AX MAXTOOL
syn keyword krlEnum INDIVIDUAL_MAMES SUPPLY_VOLTAGE KINCLASS MAIN_AXIS WRIST_AXIS SW_ONOFF
"
" Predefined structures and enums found in /r1/mada/$robcor.dat
" syn keyword krlStructure
syn keyword krlEnum ADAP_ACC MODEL_TYPE CONTROL_PARAMETER EKO_MODE
"
" Predefined structures and enums found in /steu/mada/$custom.dat
syn keyword krlStructure PRO_IO_T SER EXT_MOD_T COOP_KRC WS_CONFIG BIN_TYPE COOP_UPDATE_T LDC_REACTION
syn keyword krlEnum AXIS_OF_COORDINATES SPLINE_PARA_VARIANT TARGET_STATUS CP_VEL_TYPE CP_STATMON
"
" Predefined structures and enums found in /steu/mada/$machine.dat
syn keyword krlStructure EMSTOP_PATH BOXSTATESAFEIN BOXSTATESAFEOUT
syn keyword krlEnum DIGINCODE
"
" Predefined structures and enums found in /steu/mada/$option.dat
syn keyword krlStructure MSG_T
" syn keyword krlEnum
"
" Predefined structures and enums found in /r1/system/$config.dat
" BasisTech
syn keyword krlStructure DIG_OUT_TYPE CTRL_IN_T CTRL_OUT_T FCT_OUT_T FCT_IN_T ODAT BASIS_SUGG_T OUT_SUGG_T MACHINE_DEF_T MACHINE_TOOL_T MACHINE_FRAME_T TRIGGER_PARA CONSTVEL_PARA CONDSTOP_PARA ADAT TM_SUGG_T TQM_TQDAT_T SPS_PROG_TYPE
syn keyword krlEnum BAS_COMMAND OUT_MODETYPE IPO_M_T APO_MODE_T FUNCT_TYPE P00_COMMAND
"
" GripperTech
syn keyword krlStructure GRP_TYP GRP_TYPES GRP_SUGG_T
syn keyword krlEnum ON_OFF_TYP APO_TYP " COMMAND
"
" SpotTech
syn keyword krlStructure SPOT_TYPE SPOT_SUGG_T
syn keyword krlEnum S_COMMAND S_PAIR_SLCT
"
" VW
syn keyword krlStructure VW_MPARA_TYP ZANGENTYP ZANGENBEDINGUNG IBSZANGENTYP LAST_IBS_TYP VERR_TYP VERRCHECK_T T_FB_STATE Kollisionsdaten State_T MODUS_T
syn keyword krlEnum SYNCTYPE DIR_TYP SUBTYPE ARI_TYP BOOL_TYP VW_COMMAND IBGN_COMMAND VW_USER_CMD MOVE_TYPES ADV_T_TYPE BAS_TYPE IBS_MODE_TYP VW_USER_CMD PRO_MODE MODE_OP
"
" ProgCoop
syn keyword krlStructure YDAT
" syn keyword krlEnum
"
" bas.src
syn keyword krlStructure CONT 
syn keyword krlEnum ESYS IPO_MODE CIRC_MODE CIRC_TYPE ORI_TYPE VAR_STATE
"
" MsgLib.src
syn keyword krlStructure KrlMsg_T KrlMsgParType_T KrlMsgPar_T KrlMsgOpt_T KrlMsgDlgSK_T
syn keyword krlEnum EKrlMsgType
"
highlight link krlStructure Structure
highlight link krlEnum Structure
" ---

" System variable
syn match krlSysvars /\$\a[a-zA-Z0-9_.]*/
highlight link krlSysvars Sysvars
" ---

" Statement
syn keyword krlStatement WAIT SEC INTERRUPT ON OFF TRIGGER WHEN DISTANCE PATH DELAY DO PRIO CONTINUE IMPORT IS MINIMUM MAXIMUM CONFIRM ON_ERROR_PROCEED
highlight link krlStatement Statement
" Conditional
syn keyword krlConditional IF THEN ELSE ENDIF SWITCH CASE DEFAULT ENDSWITCH
highlight link krlConditional Conditional
" Repeat
syn keyword krlRepeat FOR TO STEP ENDFOR WHILE ENDWHILE REPEAT UNTIL LOOP ENDLOOP EXIT 
highlight link krlRepeat Repeat
" Label
syn keyword krlLabel GOTO
syn match krlLabel /^\s*\a\w\+:/
highlight link krlLabel Label
" Keyword
syn keyword krlKeyword ANIN ANOUT DIGIN
highlight link krlKeyword Keyword
" Exception
syn keyword krlException RETURN RESUME HALT
highlight link krlException Exception
" ---

" special keywords for movement commands
syn keyword krlMovement PTP LIN CIRC ASYPTP PTP_REL LIN_REL CIRC_REL
syn keyword krlMovement ASYCANCEL BRAKE BRAKE_F
highlight link krlMovement Movement
" movement modifiers
syn keyword krlMoveMod CA C_PTP C_DIS C_VEL C_ORI
highlight link krlMoveMod Movement
" ---

" Operator
" Boolean operator
syn keyword krlBoolOperator AND OR EXOR NOT DIV MOD B_AND B_OR B_EXOR
highlight link krlBoolOperator Operator
" Arithmetic operator
syn match krlArithOperator /[+-]/ containedin=krlFloat
syn match krlArithOperator /\*\|\//
highlight link krlArithOperator Operator
" Compare operator
syn match krlCompOperator /[<>=]/
highlight link krlCompOperator Operator
" Geometric operator
" Do not move the : operator
" Must be present befor krlParamdef
syn match krlGeomOperator /[:]/ " containedin=krlParamdef
highlight link krlGeomOperator Operator
" ---

" Type
" Simple data types
syn keyword krlType BOOL CHAR REAL INT
" Enumeration and structure
syn keyword krlType ENUM STRUC
" External program and function
syn keyword krlType EXT EXTFCT
" Communication
syn keyword krlType SIGNAL CHANNEL
highlight link krlType Type
" StorageClass
syn keyword krlStorageClass DECL GLOBAL PUBLIC CONST
highlight link krlStorageClass StorageClass
" Parameter StorageClass
" Do not move the :in/:out
" Must be present after krlGeomOperator
" This will generate false highlight if a frame is called "in" or "out"
" I tried, but don't know what to do about this
syn match krlParamdef /[:] *in\>/
syn match krlParamdef /[:] *out\>/
highlight link krlParamdef StorageClass
" Not a typedef but I like to have those highlighted 
" different then types, structures or strorage classes
syn keyword krlTypedef DEF END DEFFCT ENDFCT DEFDAT ENDDAT
highlight link krlTypedef Typedef
" ---

" Delimiter
syn match krlDelimiter /\\\||\|\[\|\]\|[()]\|[,]/
highlight link krlDelimiter Delimiter
" ---

" avoid coloring structure component names
" if they have the same name as a type
syn match krlNames /[a-zA-Z_][.a-zA-Z0-9_]*/ containedin=krlStructVal
highlight link krlNames None
" Structure value
syn region krlStructVal start=/{/ end=/}/ containedin=krlStructVal
highlight link krlStructVal krlDelimiter
" ---

" BuildInFunction
syn keyword krlBuildInFunction contained abs sin cos acos tan atan atan2 sqrt
syn keyword krlBuildInFunction contained b_not " maybe this one should move to Operator?! It's used like a function: b_not(bool)
syn keyword krlBuildInFunction contained cClose cOpen cRead cWrite sRead sWrite
syn keyword krlBuildInFunction contained forward inverse inv_pos
syn keyword krlBuildInFunction contained get_sig_inf pulse
syn keyword krlBuildInFunction contained StrAdd StrClear StrCopy StrComp StrFind StrLen StrDeclLen StrToBool StrToInt StrToReal StrToString
syn keyword krlBuildInFunction contained Set_KrlDlg Exists_KrlDlg Set_KrlMsg Exists_KrlMsg
syn keyword krlBuildInFunction contained Err_Clear Err_Raise
syn keyword krlBuildInFunction contained varstate EK EB LK sync MD_CMD
highlight link krlBuildInFunction BuildInFunction
" ---

" Function
syn match krlFunction /[a-zA-Z_]\w* *(/me=e-1 contains=krlBuildInFunction
highlight link krlFunction Function
" ---

" Error
" TODO highlight false use of = and == as Error
" syn match krlError /\(^\s*\(wait\|if\|while\|until\|\(global\s\+\)\?interrupt\)\s\+\)\@<=[^;\<\>=]\+=[^=]/
" syn match krlError /\(^\s*\(wait\|if\|while\|until\|\(global\s\+\)\?interrupt\).*\)\@<=[^;\<\>=]\+=[^=]/
" syn match krlError /^\(\s*\(wait\|if\|while\|until\|\(global\s\+\)\?interrupt\|[^;]\+[^=;]=[^=]\)\)\@![^;]*==/
highlight link krlError Error
" ---

let b:current_syntax = "krl"

